/*
  camlib.c
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include "camlib.h"

#define DEVNAME "/dev/k2917"

static int fdcam=-1;
extern int errno;

int CAM_Open( void )
{
  if(fdcam!=-1)
    close(fdcam);
  if((fdcam=open(DEVNAME,O_RDWR))<0){
    fprintf(stderr,"COPEN: open failure\n");
    return 1;
  }
  return 0; 
}

int CAM_Close( void )
{
  if(fdcam<0){
    fprintf(stderr,"CCLOSE: not opend file\n");
    return EBADF;
  }
  if(close(fdcam)){
    fprintf(stderr,"CCLOSE: close failure\n");
    return errno;
  }
  fdcam=-1;
  return 0;
}

int CRESET( void )
{
  ioctl(fdcam,K2917_IOC_RESET);
  return 0;
}

int CSETBR( int br )
{
  return 0;
}

int CSETCR( int cr )
{
  if(cr>=0 && cr<8)
    if(ioctl(fdcam,K2917_IOC_SETCR,(unsigned long)cr))
      return errno;
    else
      return 0;
  else
    return EINVAL;
}

int CGENZ( void )
{
  int status,q,x;
  short data=0;
  if((status=CAMACW(NAF(30,0,1),&data,&q,&x))!=0)
    return status;
  data=(data|0x1)&K3922SR_WMASK;
  return(CAMACW(NAF(30,0,17),&data,&q,&x));
}

int CGENC( void )
{
  int status,q,x;
  short data=0;
  if((status=CAMACW(NAF(30,0,1),&data,&q,&x))!=0)
    return status;
  data=(data|0x2)&K3922SR_WMASK;
  return(CAMACW(NAF(30,0,17),&data,&q,&x));
}

int CSETI( void )
{
  int status,q,x;
  short data=0;
  if((status=CAMACW(NAF(30,0,1),&data,&q,&x))!=0)
    return status;
  data=(data|0x4)&K3922SR_WMASK;
  return(CAMACW(NAF(30,0,17),&data,&q,&x));
}

int CREMI( void )
{
  int status,q,x;
  short data=0;
  if((status=CAMACW(NAF(30,0,1),&data,&q,&x))!=0)
    return status;
  data=data&(~0x4)&K3922SR_WMASK;
  return(CAMACW(NAF(30,0,17),&data,&q,&x));
}

int CAMACL( int naf, int *data, int *q, int *x )
{
  struct k2917_single single;
  int status=0;

  single.naf=naf; single.q=single.x=0;
  if(NAFCHKF8(naf)){         /* Control Operation */
    if(ioctl(fdcam,K2917_IOC_CAMC,&single))
      status=errno;
    *q=single.q; *x=single.x;
  }
  else if(NAFCHKF16(naf)){ /* Write Operation */
    single.data=(*data);
    if(ioctl(fdcam,K2917_IOC_CAMW24,&single))
      status=errno;
    *q=single.q; *x=single.x;
  }
  else{                    /* Read Operation */
    if(ioctl(fdcam,K2917_IOC_CAMR24,&single))
      status=errno;
    *data=single.data&0xFFFFFF;
    *q=single.q; *x=single.x;
  }
  return status;
} 

int CAMACW( int naf, short *data, int *q, int *x )
{
  struct k2917_single single;
  int status=0;

  single.naf=naf; single.q=single.x=0;
  if(NAFCHKF8(naf)){         /* Control Operation */
    if(ioctl(fdcam,K2917_IOC_CAMC,&single))
      status=errno;
    *q=single.q; *x=single.x;
  }
  else if(NAFCHKF16(naf)){ /* Write Operation */
    single.data=(int)((*data)&0xFFFF);
    if(ioctl(fdcam,K2917_IOC_CAMW16,&single))
      status=errno;
    *q=single.q; *x=single.x;
  }
  else{                    /* Read Operation */
    if(ioctl(fdcam,K2917_IOC_CAMR16,&single))
      status=errno;
    *data=(short)(single.data&0xFFFF);
    *q=single.q; *x=single.x;
  }
  return status;
} 

int CAM_EnableLAM( int mask )
{
  if(ioctl(fdcam,K2917_IOC_EI,(unsigned long)mask))
    return errno;
  return 0;
}

int CAM_DisableLAM( void )
{
  if(ioctl(fdcam,K2917_IOC_DI))
    return errno;
  return 0;
}

int CAM_WaitLAM( int timeout )
{
  int status;
  status=ioctl(fdcam,K2917_IOC_WLAM,(unsigned long)timeout);
  if(status){
#ifdef DEBUGINT
    fprintf(stderr,"WaitLAM: Status=%d ErrNo=%d\n",status,errno);
#endif
    return errno;
  }
  return 0;
}

int CAMRegisterList( int id, int size, int bsize, int *action_list )
{
  struct k2917_al_struct al;

  al.al_num=id;
  al.size=size;
  al.buffer_size=bsize;
  al.action_list=action_list;
  if(ioctl(fdcam,K2917_IOC_ALREG,&al,(unsigned long)id))
    return errno;
  return 0;
}

int CAMUnregisterList( int id )
{
  if(ioctl(fdcam,K2917_IOC_ALUNREG,(unsigned long)id))
    return errno;
  return 0;
}

int CAMBlockRead( int id, int maxsize, int *buffer )
{
  struct k2917_ad_struct ad;
  int retval;

  ad.al_num=id;
  ad.maxsize=maxsize;
  ad.size=0;
  ad.buffer=buffer;
  retval=ioctl(fdcam,K2917_IOC_EXEC_LA3,&ad);
#ifdef DEBUGLIST
  fprintf(stderr,"CAMBlockRead: RetVal=%d ErrNo=%d Size=%d\n",
	  retval,errnor,ad.size);
#endif
  return ad.size;
}
